@@ -12,6 +12,7 @@ module Hledger.Cli.CliOptions (
   -- * cmdargs flags & modes
   helpflags,
   detailedversionflag,
+  hiddenflags,
   inputflags,
   reportflags,
   outputflags,
@@ -34,7 +35,6 @@ module Hledger.Cli.CliOptions (
   CliOpts(..),
   defcliopts,
   getHledgerCliOpts,
-  decodeRawOpts,
   rawOptsToCliOpts,
   checkCliOpts,
   outputFormats,
@@ -112,7 +112,7 @@ helpflags = [
  ,flagNone ["version"] (setboolopt "version") "show version information"
  ]
 
--- | A hidden flag, just for the hledger executable.
+-- | A hidden flag just for the hledger executable.
 detailedversionflag :: Flag RawOpts
 detailedversionflag = flagNone ["version+"] (setboolopt "version+") "show version information with extra detail"
 
@@ -121,7 +121,7 @@ inputflags :: [Flag RawOpts]
 inputflags = [
   flagReq  ["file","f"]      (\s opts -> Right $ setopt "file" s opts) "FILE" "use a different input file. For stdin, use - (default: $LEDGER_FILE or $HOME/.hledger.journal)"
  ,flagReq  ["rules-file"]    (\s opts -> Right $ setopt "rules-file" s opts) "RFILE" "CSV conversion rules file (default: FILE.rules)"
- ,flagReq  ["separator"]     (\s opts -> Right $ setopt "separator" s opts) "SEPARATOR" "CSV separator (default: ,)"
+ ,flagReq  ["separator"]     (\s opts -> Right $ setopt "separator" s opts) "SEP" "CSV separator (default: ,)"
  ,flagReq  ["alias"]         (\s opts -> Right $ setopt "alias" s opts)  "OLD=NEW" "rename accounts named OLD to NEW"
  ,flagNone ["anon"]          (setboolopt "anon") "anonymize accounts and payees"
  ,flagReq  ["pivot"]         (\s opts -> Right $ setopt "pivot" s opts)  "TAGNAME" "use some other field/tag for account names"
@@ -131,6 +131,8 @@ inputflags = [
 -- | Common report-related flags: --period, --cost, etc.
 reportflags :: [Flag RawOpts]
 reportflags = [
+
+  -- report period & interval
   flagReq  ["begin","b"]     (\s opts -> Right $ setopt "begin" s opts) "DATE" "include postings/txns on or after this date"
  ,flagReq  ["end","e"]       (\s opts -> Right $ setopt "end" s opts) "DATE" "include postings/txns before this date"
  ,flagNone ["daily","D"]     (setboolopt "daily") "multiperiod/multicolumn report by day"
@@ -138,19 +140,54 @@ reportflags = [
  ,flagNone ["monthly","M"]   (setboolopt "monthly") "multiperiod/multicolumn report by month"
  ,flagNone ["quarterly","Q"] (setboolopt "quarterly") "multiperiod/multicolumn report by quarter"
  ,flagNone ["yearly","Y"]    (setboolopt "yearly") "multiperiod/multicolumn report by year"
- ,flagReq  ["period","p"]    (\s opts -> Right $ setopt "period" s opts) "PERIODEXP" "set start date, end date, and/or report interval all at once (overrides the flags above)"
- ,flagNone ["date2"]         (setboolopt "date2") "match the secondary date instead (see command help for other effects)"
+ ,flagReq  ["period","p"]    (\s opts -> Right $ setopt "period" s opts) "PERIODEXP" "set start date, end date, and/or report interval all at once"
+ ,flagNone ["date2"]         (setboolopt "date2") "match the secondary date instead. See command help for other effects. (--effective, --aux-date also accepted)"  -- see also hiddenflags
 
+  -- status/realness/depth/zero filters
  ,flagNone ["unmarked","U"]  (setboolopt "unmarked") "include only unmarked postings/txns (can combine with -P or -C)"
  ,flagNone ["pending","P"]   (setboolopt "pending") "include only pending postings/txns"
  ,flagNone ["cleared","C"]   (setboolopt "cleared") "include only cleared postings/txns"
  ,flagNone ["real","R"]      (setboolopt "real") "include only non-virtual postings"
  ,flagReq  ["depth"]         (\s opts -> Right $ setopt "depth" s opts) "NUM" "(or -NUM): hide accounts/postings deeper than this"
  ,flagNone ["empty","E"]     (setboolopt "empty") "show items with zero amount, normally hidden (and vice-versa in hledger-ui/hledger-web)"
- ,flagNone ["cost","B"]      (setboolopt "cost") "convert amounts to their cost at transaction time (using the transaction price, if any)"
- ,flagNone ["value","V"]     (setboolopt "value") "convert amounts to their market value on the report end date (using the most recent applicable market price, if any)"
+
+  -- valuation
+ ,flagNone ["B","cost"]      (setboolopt "B")
+   "show amounts converted to their cost, using the transaction price. Equivalent to --value=cost."
+ ,flagNone ["V","market"]    (setboolopt "V")
+   (unwords
+     ["show amounts converted to current market value (single period reports)"
+     ,"or period-end market value (multiperiod reports) in their default valuation commodity."
+     ,"Equivalent to --value=now / --value=end."
+     ])
+ ,flagReq ["X","exchange"]   (\s opts -> Right $ setopt "X" s opts) "COMM"
+   (unwords
+     ["show amounts converted to current (single period reports)"
+     ,"or period-end (multiperiod reports) market value in the specified commodity."
+     ,"Equivalent to --value=now,COMM / --value=end,COMM."
+     ])
+ ,flagReq  ["value"]         (\s opts -> Right $ setopt "value" s opts) "TYPE[,COMM]"
+   (unlines
+     ["TYPE is cost, end, now or YYYY-MM-DD."
+     ,"COMM is an optional commodity symbol."
+     ,"Shows amounts converted to:"
+     ,"- cost using transaction prices, then optionally to COMM using period-end market prices"
+     ,"- period-end market value, in default valuation commodity or COMM"
+     ,"- current market value, in default valuation commodity or COMM"
+     ,"- market value on the given date, in default valuation commodity or COMM"
+     ])
+
+  -- generated postings/transactions
  ,flagNone ["auto"]          (setboolopt "auto") "apply automated posting rules to modify transactions"
  ,flagNone ["forecast"]      (setboolopt "forecast") "apply periodic transaction rules to generate future transactions, to 6 months from now or report end date"
+
+ ]
+
+-- | Common flags that are accepted but not shown in --help,
+-- such as --effective, --aux-date.
+hiddenflags :: [Flag RawOpts]
+hiddenflags = [
+  flagNone ["effective","aux-date"] (setboolopt "date2") "Ledger-compatible aliases for --date2"
  ]
 
 -- | Common output-related flags: --output-file, --output-format...
@@ -186,7 +223,7 @@ defMode = Mode {
    ,groupHidden  = []             --  flags not displayed in the usage
    }
  ,modeArgs        = ([], Nothing) -- description of arguments accepted by the command
- ,modeValue       = []            -- value returned when this mode is used to parse a command line
+ ,modeValue       = def           -- value returned when this mode is used to parse a command line
  ,modeCheck       = Right         -- whether the mode's value is correct
  ,modeReform      = const Nothing -- function to convert the value back to a command line arguments
  ,modeExpandAt    = True          -- expand @ arguments for program ?
@@ -208,7 +245,7 @@ defCommandMode names = defMode {
     ,groupHidden  = []             --  flags not displayed in the usage
     }
   ,modeArgs = ([], Just $ argsFlag "[QUERY]")
-  ,modeValue=[("command", headDef "" names)]
+  ,modeValue=setopt "command" (headDef "" names) def
   }
 
 -- | A cmdargs mode representing the hledger add-on command with the
@@ -231,8 +268,8 @@ addonCommandMode name = (defCommandMode [name]) {
      --  ]
   ,modeGroupFlags = Group {
       groupUnnamed = []
-     ,groupHidden = []
-     ,groupNamed = [generalflagsgroup1]
+     ,groupHidden  = hiddenflags
+     ,groupNamed   = [generalflagsgroup1]
      }
   }
 
@@ -244,7 +281,7 @@ type CommandDoc = String
 -- from a help template and flag/argument specifications.
 -- Reduces boilerplate a little, though the complicated cmdargs
 -- flag and argument specs are still required.
-hledgerCommandMode :: CommandDoc -> [Flag RawOpts] -> [(String, [Flag RawOpts])] 
+hledgerCommandMode :: CommandDoc -> [Flag RawOpts] -> [(String, [Flag RawOpts])]
   -> [Flag RawOpts] -> ([Arg RawOpts], Maybe (Arg RawOpts)) -> Mode RawOpts
 hledgerCommandMode doc unnamedflaggroup namedflaggroups hiddenflaggroup argsdescr =
   case parseCommandDoc doc of
@@ -265,9 +302,9 @@ hledgerCommandMode doc unnamedflaggroup namedflaggroups hiddenflaggroup argsdesc
 --
 -- - First line: the command name then any aliases, as one or more space or comma-separated words
 --
--- - Second line to a line containing just _FLAGS_, or the end: the short help
+-- - Second line to a line containing just _FLAGS, or the end: the short help
 --
--- - Any lines after _FLAGS_: the long help (split into lines for cmdargs)
+-- - Any lines after _FLAGS: the long help (split into lines for cmdargs)
 --
 -- The CLI help displays the short help, then the cmdargs-generated
 -- flags list, then the long help (which some day we might make
@@ -281,7 +318,7 @@ parseCommandDoc t =
     (l:ls) -> Just (names, shorthelp, longhelplines)
       where
         names = words $ map (\c -> if c `elem` [',','\\'] then ' ' else c) l
-        (shorthelpls, longhelpls) = break (== "_FLAGS_") ls
+        (shorthelpls, longhelpls) = break (== "_FLAGS") ls
         shorthelp = unlines $ reverse $ dropWhile null $ reverse shorthelpls
         longhelplines = dropWhile null $ drop 1 longhelpls
 
@@ -291,13 +328,12 @@ showModeUsage = (showText defaultWrap :: [Text] -> String) .
                (helpText [] HelpFormatDefault :: Mode a -> [Text])
 
 -- | Get the most appropriate documentation topic for a mode.
--- Currently, that is either the hledger, hledger-ui, hledger-web or
--- hledger-api manual.
+-- Currently, that is either the hledger, hledger-ui or hledger-web
+-- manual.
 topicForMode :: Mode a -> Topic
 topicForMode m
   | n == "hledger-ui"  = "ui"
   | n == "hledger-web" = "web"
-  --  | n == "hledger-api" = lookupDocTxt "api" -- hledger-api uses docopt
   | otherwise          = "cli"
   where n = headDef "" $ modeNames m
 
@@ -357,16 +393,12 @@ defcliopts = CliOpts
     def
     defaultWidth
 
--- | Convert possibly encoded option values to regular unicode strings.
-decodeRawOpts :: RawOpts -> RawOpts
-decodeRawOpts = map (\(name',val) -> (name', fromSystemString val))
-
 -- | Default width for hledger console output, when not otherwise specified.
 defaultWidth :: Int
 defaultWidth = 80
 
 -- | Replace any numeric flags (eg -2) with their long form (--depth 2),
--- as I'm guessing cmdargs doesn't support this directly.  
+-- as I'm guessing cmdargs doesn't support this directly.
 replaceNumericFlags :: [String] -> [String]
 replaceNumericFlags = map replace
   where
@@ -393,7 +425,7 @@ rawOptsToCliOpts rawopts = checkCliOpts <$> do
   return defcliopts {
               rawopts_         = rawopts
              ,command_         = stringopt "command" rawopts
-             ,file_            = map (T.unpack . stripquotes . T.pack) $ listofstringopt "file" rawopts
+             ,file_            = listofstringopt "file" rawopts
              ,inputopts_       = iopts
              ,reportopts_      = ropts
              ,output_file_     = maybestringopt "output-file" rawopts
@@ -414,10 +446,10 @@ checkCliOpts opts =
       Right _  -> Right ()
   -- XXX check registerWidthsFromOpts opts
 
--- | A helper for addon commands: this parses options and arguments from 
--- the current command line using the given hledger-style cmdargs mode, 
--- and returns a CliOpts. Or, with --help or -h present, it prints 
--- long or short help, and exits the program. 
+-- | A helper for addon commands: this parses options and arguments from
+-- the current command line using the given hledger-style cmdargs mode,
+-- and returns a CliOpts. Or, with --help or -h present, it prints
+-- long or short help, and exits the program.
 -- When --debug is present, also prints some debug output.
 -- Note this is not used by the main hledger executable.
 --
@@ -434,13 +466,13 @@ checkCliOpts opts =
 -- hledger options not displayed.
 --
 -- Tips:
--- Empty lines in the pre/postamble are removed by cmdargs; 
+-- Empty lines in the pre/postamble are removed by cmdargs;
 -- add a space character to preserve them.
 --
 getHledgerCliOpts :: Mode RawOpts -> IO CliOpts
 getHledgerCliOpts mode' = do
   args' <- getArgs >>= expandArgsAt
-  let rawopts = either usageError decodeRawOpts $ process mode' args'
+  let rawopts = either usageError id $ process mode' args'
   opts <- rawOptsToCliOpts rawopts
   debugArgs args' opts
   when ("help" `inRawOpts` rawopts_ opts) $ putStr shorthelp >> exitSuccess
@@ -602,7 +634,7 @@ defaultBalanceLineFormat = BottomAligned [
 -- or more versions (or two versions that don't look like a
 -- source/compiled pair), they are all included, with file extensions
 -- intact.
--- 
+--
 hledgerAddons :: IO [String]
 hledgerAddons = do
   -- past bug generator
@@ -620,10 +652,10 @@ dropRedundantSourceVersion [f,g]
   | takeExtension g `elem` compiledExts = [g]
 dropRedundantSourceVersion fs = fs
 
-compiledExts = ["",".com",".exe"] 
+compiledExts = ["",".com",".exe"]
 
 
--- | Get all sorted unique filenames in the current user's PATH. 
+-- | Get all sorted unique filenames in the current user's PATH.
 -- We do not currently filter out non-file objects or files without execute permission.
 likelyExecutablesInPath :: IO [String]
 likelyExecutablesInPath = do
@@ -639,8 +671,8 @@ likelyExecutablesInPath = do
 
 -- | Get the sorted unique filenames of all hledger-* executables in
 -- the current user's PATH. These are files in any of the PATH directories,
--- named hledger-*, with either no extension (and no periods in the name) 
--- or one of the addonExtensions. 
+-- named hledger-*, with either no extension (and no periods in the name)
+-- or one of the addonExtensions.
 -- We do not currently filter out non-file objects or files without execute permission.
 hledgerExecutablesInPath :: IO [String]
 hledgerExecutablesInPath = filter isHledgerExeName <$> likelyExecutablesInPath
