@@ -16,7 +16,7 @@ module Hledger.Cli.Commands (
   ,builtinCommands
   ,builtinCommandNames
   ,printCommandsList
-  ,tests_Commands
+  ,tests_Hledger_Cli
   ,module Hledger.Cli.Commands.Accounts
   ,module Hledger.Cli.Commands.Activity
   ,module Hledger.Cli.Commands.Add
@@ -28,9 +28,13 @@ module Hledger.Cli.Commands (
   ,module Hledger.Cli.Commands.Checkdupes
   ,module Hledger.Cli.Commands.Close
   ,module Hledger.Cli.Commands.Commodities
+  ,module Hledger.Cli.Commands.Descriptions
+  ,module Hledger.Cli.Commands.Diff
   ,module Hledger.Cli.Commands.Help
   ,module Hledger.Cli.Commands.Import
   ,module Hledger.Cli.Commands.Incomestatement
+  ,module Hledger.Cli.Commands.Notes
+  ,module Hledger.Cli.Commands.Payees
   ,module Hledger.Cli.Commands.Prices
   ,module Hledger.Cli.Commands.Print
   ,module Hledger.Cli.Commands.Printunique
@@ -51,9 +55,9 @@ import Data.Monoid ((<>))
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Time.Calendar
-import qualified EasyTest
+import System.Environment (withArgs)
 import System.Console.CmdArgs.Explicit as C
-import System.Exit
+import Test.Tasty (defaultMain)
 
 import Hledger 
 import Hledger.Cli.CliOptions
@@ -69,10 +73,14 @@ import Hledger.Cli.Commands.Checkdates
 import Hledger.Cli.Commands.Checkdupes
 import Hledger.Cli.Commands.Close
 import Hledger.Cli.Commands.Commodities
+import Hledger.Cli.Commands.Descriptions
+import Hledger.Cli.Commands.Diff
 import Hledger.Cli.Commands.Files
 import Hledger.Cli.Commands.Help
 import Hledger.Cli.Commands.Import
 import Hledger.Cli.Commands.Incomestatement
+import Hledger.Cli.Commands.Notes
+import Hledger.Cli.Commands.Payees
 import Hledger.Cli.Commands.Prices
 import Hledger.Cli.Commands.Print
 import Hledger.Cli.Commands.Printunique
@@ -82,6 +90,7 @@ import Hledger.Cli.Commands.Rewrite
 import Hledger.Cli.Commands.Roi
 import Hledger.Cli.Commands.Stats
 import Hledger.Cli.Commands.Tags
+import Hledger.Cli.Utils (tests_Cli_Utils)
 
 -- | The cmdargs subcommand mode (for command-line parsing)
 -- and IO action (for doing the command's work) for each builtin command.
@@ -99,10 +108,14 @@ builtinCommands = [
   ,(checkdupesmode         , checkdupes)
   ,(closemode              , close)
   ,(commoditiesmode        , commodities)
+  ,(descriptionsmode        , descriptions)
   ,(helpmode               , help')
   ,(importmode             , importcmd)
   ,(filesmode              , files)
+  ,(diffmode               , diff)
   ,(incomestatementmode    , incomestatement)
+  ,(notesmode              , notes)
+  ,(payeesmode             , payees)
   ,(pricesmode             , prices)
   ,(printmode              , print')
   ,(printuniquemode        , printunique)
@@ -151,7 +164,7 @@ commandsList = unlines [
   ," check-dates              check transactions are ordered by date"
   ," check-dupes              check for accounts with the same leaf name"
   ," close (equity)           generate balance-resetting transactions"
-  ,"+diff                     compare account transactions in two journal files"
+  ," diff                     compare account transactions in two journal files"
   ,"+interest                 generate interest transactions"
   ," rewrite                  generate automated postings/diffs (old, use --auto)"
   ,""
@@ -168,7 +181,10 @@ commandsList = unlines [
   ," activity                 show postings-per-interval bar charts"
   ," balance (b, bal)         show balance changes/end balances/budgets in accounts"
   ," commodities              show commodity/currency symbols"
+  ," descriptions             show unique transaction descriptions"
   ," files                    show input file paths"
+  ," notes                    show unique note segments of transaction descriptions"
+  ," payees                   show unique payee segments of transaction descriptions"
   ," prices                   show market price records"
   ," print (p, txns)          show transactions (journal entries)"
   ," print-unique             show only transactions with unique descriptions"
@@ -243,88 +259,98 @@ testmode = hledgerCommandMode
   []
   [generalflagsgroup3]
   []
-  ([], Just $ argsFlag "[TESTPATTERN] [SEED]")
+  ([], Just $ argsFlag "[-- TASTYOPTS]")
 
--- | The test command.
+-- | The test command, which runs the hledger and hledger-lib
+-- packages' unit tests. This command also accepts tasty test runner
+-- options, written after a -- (double hyphen).
+--
 -- Unlike most hledger commands, this one does not read the user's journal.
 -- A 'Journal' argument remains in the type signature, but it should
 -- not be used (and would raise an error).
+--
 testcmd :: CliOpts -> Journal -> IO ()
 testcmd opts _undefined = do 
-  let args = words' $ query_ $ reportopts_ opts
-  -- workaround for https://github.com/joelburget/easytest/issues/11 
---  import System.IO (hSetEncoding, stdout, stderr, utf8)
---  hSetEncoding stdout utf8
---  hSetEncoding stderr utf8
-  e <- runEasytests args $ EasyTest.tests [tests_Hledger, tests_Commands]
-  if e then exitFailure else exitSuccess
+  withArgs (words' $ query_ $ reportopts_ opts) $
+    Test.Tasty.defaultMain $ tests "hledger" [
+       tests_Hledger
+      ,tests_Hledger_Cli
+      ]
 
-
--- unit tests of hledger command-line executable
+-- All unit tests for Hledger.Cli, defined here rather than
+-- Hledger.Cli so testcmd can use them.
+tests_Hledger_Cli = tests "Hledger.Cli" [
+   tests_Cli_Utils
+  ,tests_Commands
+  ]
 
 tests_Commands = tests "Commands" [
    tests_Balance
   ,tests_Register
 
   -- some more tests easiest to define here:
-  
-  ,test "apply account directive" $ do 
-    let ignoresourcepos j = j{jtxns=map (\t -> t{tsourcepos=nullsourcepos}) (jtxns j)}
-    let sameParse str1 str2 = do j1 <- io $ readJournal def Nothing str1 >>= either error' (return . ignoresourcepos)
-                                 j2 <- io $ readJournal def Nothing str2 >>= either error' (return . ignoresourcepos)
-                                 j1 `is` j2{jlastreadtime=jlastreadtime j1, jfiles=jfiles j1} --, jparsestate=jparsestate j1}
-    sameParse
-     ("2008/12/07 One\n  alpha  $-1\n  beta  $1\n" <>
-      "apply account outer\n2008/12/07 Two\n  aigh  $-2\n  bee  $2\n" <>
-      "apply account inner\n2008/12/07 Three\n  gamma  $-3\n  delta  $3\n" <>
-      "end apply account\n2008/12/07 Four\n  why  $-4\n  zed  $4\n" <>
-      "end apply account\n2008/12/07 Five\n  foo  $-5\n  bar  $5\n"
-     )
-     ("2008/12/07 One\n  alpha  $-1\n  beta  $1\n" <>
-      "2008/12/07 Two\n  outer:aigh  $-2\n  outer:bee  $2\n" <>
-      "2008/12/07 Three\n  outer:inner:gamma  $-3\n  outer:inner:delta  $3\n" <>
-      "2008/12/07 Four\n  outer:why  $-4\n  outer:zed  $4\n" <>
-      "2008/12/07 Five\n  foo  $-5\n  bar  $5\n"
-     )
 
-  ,test "apply account directive should preserve \"virtual\" posting type" $ do
-    j <- io $ readJournal def Nothing "apply account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n" >>= either error' return
+  ,tests "apply account directive" [
+     test "works" $ do
+        let
+          ignoresourcepos j = j{jtxns=map (\t -> t{tsourcepos=nullsourcepos}) (jtxns j)}
+          sameParse str1 str2 = do
+            j1 <- readJournal def Nothing str1 >>= either error' (return . ignoresourcepos)
+            j2 <- readJournal def Nothing str2 >>= either error' (return . ignoresourcepos)
+            j1 @?= j2{jlastreadtime=jlastreadtime j1, jfiles=jfiles j1} --, jparsestate=jparsestate j1}
+        sameParse
+           ("2008/12/07 One\n  alpha  $-1\n  beta  $1\n" <>
+            "apply account outer\n2008/12/07 Two\n  aigh  $-2\n  bee  $2\n" <>
+            "apply account inner\n2008/12/07 Three\n  gamma  $-3\n  delta  $3\n" <>
+            "end apply account\n2008/12/07 Four\n  why  $-4\n  zed  $4\n" <>
+            "end apply account\n2008/12/07 Five\n  foo  $-5\n  bar  $5\n"
+           )
+           ("2008/12/07 One\n  alpha  $-1\n  beta  $1\n" <>
+            "2008/12/07 Two\n  outer:aigh  $-2\n  outer:bee  $2\n" <>
+            "2008/12/07 Three\n  outer:inner:gamma  $-3\n  outer:inner:delta  $3\n" <>
+            "2008/12/07 Four\n  outer:why  $-4\n  outer:zed  $4\n" <>
+            "2008/12/07 Five\n  foo  $-5\n  bar  $5\n"
+           )
+
+    ,test "preserves \"virtual\" posting type" $ do
+      j <- readJournal def Nothing "apply account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n" >>= either error' return
+      let p = head $ tpostings $ head $ jtxns j
+      paccount p @?= "test:from"
+      ptype p @?= VirtualPosting
+    ]
+
+  ,test "alias directive" $ do
+    j <- readJournal def Nothing "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n" >>= either error' return
     let p = head $ tpostings $ head $ jtxns j
-    paccount p `is` "test:from"
-    ptype p `is` VirtualPosting
-  
-  ,test "account aliases" $ do
-    j <- io $ readJournal def Nothing "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n" >>= either error' return
-    let p = head $ tpostings $ head $ jtxns j
-    paccount p `is` "equity:draw:personal:food"
+    paccount p @?= "equity:draw:personal:food"
+
+  ,test "Y default year directive" $ do
+    j <- readJournal def Nothing defaultyear_journal_txt >>= either error' return
+    tdate (head $ jtxns j) @?= fromGregorian 2009 1 1
 
   ,test "ledgerAccountNames" $
-    ledgerAccountNames ledger7 `is`
-     ["assets","assets:cash","assets:checking","assets:saving","equity","equity:opening balances",
-      "expenses","expenses:food","expenses:food:dining","expenses:phone","expenses:vacation",
-      "liabilities","liabilities:credit cards","liabilities:credit cards:discover"]
+    (ledgerAccountNames ledger7)
+    @?=
+    ["assets","assets:cash","assets:checking","assets:saving","equity","equity:opening balances",
+     "expenses","expenses:food","expenses:food:dining","expenses:phone","expenses:vacation",
+     "liabilities","liabilities:credit cards","liabilities:credit cards:discover"]
 
   -- ,test "journalCanonicaliseAmounts" ~:
   --  "use the greatest precision" ~:
-  --   (map asprecision $ journalAmountAndPriceCommodities $ journalCanonicaliseAmounts $ journalWithAmounts ["1","2.00"]) `is` [2,2]
+  --   (map asprecision $ journalAmountAndPriceCommodities $ journalCanonicaliseAmounts $ journalWithAmounts ["1","2.00"]) @?= [2,2]
 
   -- don't know what this should do
   -- ,test "elideAccountName" ~: do
   --    (elideAccountName 50 "aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa"
-  --     `is` "aa:aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa")
+  --     @?= "aa:aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa")
   --    (elideAccountName 20 "aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa"
-  --     `is` "aa:aa:aaaaaaaaaaaaaa")
+  --     @?= "aa:aa:aaaaaaaaaaaaaa")
 
-  ,test "default year" $ do
-    j <- io $ readJournal def Nothing defaultyear_journal_txt >>= either error' return
-    tdate (head $ jtxns j) `is` fromGregorian 2009 1 1
+  ,test "show dollars" $ showAmount (usd 1) @?= "$1.00"
 
-  ,test "show dollars" $ showAmount (usd 1) `is` "$1.00"
-
-  ,test "show hours" $ showAmount (hrs 1) `is` "1.00h"
-
- ]
+  ,test "show hours" $ showAmount (hrs 1) @?= "1.00h"
 
+  ]
 
 -- test data
 
