@@ -226,7 +226,7 @@ With @-E/--empty@:
 /What to show in empty cells/
 
 An empty periodic balance report cell is one which has no corresponding postings.
-An empty cumulative/historical balance report cell is one which has no correponding
+An empty cumulative/historical balance report cell is one which has no corresponding
 or prior postings, ie the account doesn't exist yet.
 Currently, empty cells show 0.
 
@@ -234,6 +234,7 @@ Currently, empty cells show 0.
 
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ExtendedDefaultRules #-}
+{-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE NamedFieldPuns #-}
@@ -291,53 +292,38 @@ balancemode = hledgerCommandMode
    ,flagNone ["budget"] (setboolopt "budget") "show performance compared to budget goals defined by periodic transactions"
    ,flagNone ["invert"] (setboolopt "invert") "display all amounts with reversed sign"
    ,flagNone ["transpose"] (setboolopt "transpose") "transpose rows and columns"
+   ,flagNone ["percent", "%"] (setboolopt "percent") "express values in percentage of each column's total"
    ]
    ++ outputflags
   )
   [generalflagsgroup1]
-  []
+  hiddenflags
   ([], Just $ argsFlag "[QUERY]")
 
 -- | The balance command, prints a balance report.
 balance :: CliOpts -> Journal -> IO ()
-balance opts@CliOpts{rawopts_=rawopts,reportopts_=ropts} j = do
+balance opts@CliOpts{rawopts_=rawopts,reportopts_=ropts@ReportOpts{..}} j = do
   d <- getCurrentDay
   case lineFormatFromOpts ropts of
     Left err -> error' $ unlines [err]
     Right _ -> do
-      let format   = outputFormatFromOpts opts
-          budget   = boolopt "budget" rawopts
-          interval = interval_ ropts
-      case (budget, interval) of
-        (True, _) -> do
-          -- single or multicolumn budget report
-          reportspan <- reportSpan j ropts
-          let budgetreport     = dbg1 "budgetreport"     $ budgetReport ropts assrt reportspan d j
-                where
-                  assrt          = not $ ignore_assertions_ $ inputopts_ opts
-              render = case format of
-                "csv"  -> const $ error' "Sorry, CSV output is not yet implemented for this kind of report."  -- TODO
-                "html" -> const $ error' "Sorry, HTML output is not yet implemented for this kind of report."  -- TODO
-                _      -> budgetReportAsText ropts
-          writeOutput opts $ render budgetreport
-          
-        (False, NoInterval) -> do
-          -- single column balance report
-          let report
-                | balancetype_ ropts `elem` [HistoricalBalance, CumulativeChange]
-                  = let ropts' | flat_ ropts = ropts
-                               | otherwise   = ropts{accountlistmode_=ALTree}
-                    in balanceReportFromMultiBalanceReport ropts' (queryFromOpts d ropts) j
-                          -- for historical balances we must use balanceReportFromMultiBalanceReport (also forces --no-elide)
-                | otherwise = balanceReport ropts (queryFromOpts d ropts) j -- simple Ledger-style balance report 
-              render = case format of
-                "csv"  -> \ropts r -> (++ "\n") $ printCSV $ balanceReportAsCsv ropts r
-                "html" -> \_ _ -> error' "Sorry, HTML output is not yet implemented for this kind of report."  -- TODO
-                _      -> balanceReportAsText
-          writeOutput opts $ render ropts report
-          
-        _  -> do
-          -- multi column balance report
+      let budget      = boolopt "budget" rawopts
+          multiperiod = interval_ /= NoInterval
+          format      = outputFormatFromOpts opts
+
+      if budget then do  -- single or multi period budget report
+        reportspan <- reportSpan j ropts
+        let budgetreport     = dbg1 "budgetreport"     $ budgetReport ropts assrt reportspan d j
+              where
+                assrt          = not $ ignore_assertions_ $ inputopts_ opts
+            render = case format of
+              "csv"  -> const $ error' "Sorry, CSV output is not yet implemented for this kind of report."  -- TODO
+              "html" -> const $ error' "Sorry, HTML output is not yet implemented for this kind of report."  -- TODO
+              _      -> budgetReportAsText ropts
+        writeOutput opts $ render budgetreport
+
+      else
+        if multiperiod then do  -- multi period balance report
           let report = multiBalanceReport ropts (queryFromOpts d ropts) j
               render = case format of
                 "csv"  -> (++ "\n") . printCSV . multiBalanceReportAsCsv ropts
@@ -345,15 +331,21 @@ balance opts@CliOpts{rawopts_=rawopts,reportopts_=ropts} j = do
                 _      -> multiBalanceReportAsText ropts
           writeOutput opts $ render report
 
--- rendering single-column balance reports
+        else do  -- single period simple balance report
+          let report
+                | balancetype_ `elem` [HistoricalBalance, CumulativeChange]
+                  = let ropts' | flat_ ropts = ropts
+                               | otherwise   = ropts{accountlistmode_=ALTree}
+                    in balanceReportFromMultiBalanceReport ropts' (queryFromOpts d ropts) j
+                          -- for historical balances we must use balanceReportFromMultiBalanceReport (also forces --no-elide)
+                | otherwise = balanceReport ropts (queryFromOpts d ropts) j -- simple Ledger-style balance report
+              render = case format of
+                "csv"  -> \ropts r -> (++ "\n") $ printCSV $ balanceReportAsCsv ropts r
+                "html" -> \_ _ -> error' "Sorry, HTML output is not yet implemented for this kind of report."  -- TODO
+                _      -> balanceReportAsText
+          writeOutput opts $ render ropts report
 
--- | Find the best commodity to convert to when asked to show the
--- market value of this commodity on the given date. That is, the one
--- in which it has most recently been market-priced, ie the commodity
--- mentioned in the most recent applicable historical price directive
--- before this date.
--- defaultValuationCommodity :: Journal -> Day -> CommoditySymbol -> Maybe CommoditySymbol
--- defaultValuationCommodity j d c = mpamount <$> commodityValue j d c
+-- rendering single-column balance reports
 
 -- | Render a single-column balance report as CSV.
 balanceReportAsCsv :: ReportOpts -> BalanceReport -> CSV
@@ -467,7 +459,7 @@ renderComponent1 opts (acctname, depth, total) (FormatField ljust min max field)
 -- and will include the final totals row unless --no-total is set.
 multiBalanceReportAsCsv :: ReportOpts -> MultiBalanceReport -> CSV
 multiBalanceReportAsCsv opts@ReportOpts{average_, row_total_} (MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
-  maybetranspose $ 
+  maybetranspose $
   ("Account" : map showDateSpan colspans
    ++ ["Total"   | row_total_]
    ++ ["Average" | average_]
@@ -490,7 +482,7 @@ multiBalanceReportAsCsv opts@ReportOpts{average_, row_total_} (MultiBalanceRepor
   where
     maybetranspose | transpose_ opts = transpose
                    | otherwise = id
-    
+
 -- | Render a multi-column balance report as HTML.
 multiBalanceReportAsHtml :: ReportOpts -> MultiBalanceReport -> Html ()
 multiBalanceReportAsHtml ropts mbr =
@@ -514,7 +506,7 @@ multiBalanceReportHtmlRows ropts mbr =
   in
     (multiBalanceReportHtmlHeadRow ropts headingsrow
     ,map (multiBalanceReportHtmlBodyRow ropts) bodyrows
-    ,multiBalanceReportHtmlFootRow ropts <$> mtotalsrow -- TODO pad totals row with zeros when there are 
+    ,multiBalanceReportHtmlFootRow ropts <$> mtotalsrow -- TODO pad totals row with zeros when there are
     )
 
 -- | Render one MultiBalanceReport heading row as a HTML table row.
@@ -557,8 +549,8 @@ multiBalanceReportHtmlBodyRow ropts (label:rest) =
 multiBalanceReportHtmlFootRow :: ReportOpts -> [String] -> Html ()
 multiBalanceReportHtmlFootRow _ropts [] = mempty
 -- TODO pad totals row with zeros when subreport is empty
---  multiBalanceReportHtmlFootRow ropts $ 
---     "" 
+--  multiBalanceReportHtmlFootRow ropts $
+--     ""
 --   : repeat nullmixedamt zeros
 --  ++ (if row_total_ ropts then [nullmixedamt] else [])
 --  ++ (if average_ ropts   then [nullmixedamt]   else [])
@@ -582,48 +574,59 @@ multiBalanceReportHtmlFootRow ropts (acct:rest) =
 
 -- | Render a multi-column balance report as plain text suitable for console output.
 multiBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> String
-multiBalanceReportAsText opts r =
-    printf "%s in %s:\n\n" desc (showDateSpan $ multiBalanceReportSpan r)
-      ++ balanceReportTableAsText opts tabl
+multiBalanceReportAsText ropts@ReportOpts{..} r =
+    title ++ "\n\n" ++ (balanceReportTableAsText ropts $ balanceReportAsTable ropts r)
   where
-    tabl = balanceReportAsTable opts r
-    desc = case balancetype_ opts of
-        PeriodChange -> "Balance changes"
-        CumulativeChange -> "Ending balances (cumulative)"
-        HistoricalBalance -> "Ending balances (historical)"
+    multiperiod = interval_ /= NoInterval
+    title = printf "%s in %s%s:"
+      (case balancetype_ of
+        PeriodChange       -> "Balance changes"
+        CumulativeChange   -> "Ending balances (cumulative)"
+        HistoricalBalance  -> "Ending balances (historical)")
+      (showDateSpan $ multiBalanceReportSpan r)
+      (case value_ of
+        Just (AtCost _mc)   -> ", valued at cost"
+        Just (AtEnd _mc)    -> ", valued at period ends"
+        Just (AtNow _mc)    -> ", current value"
+        -- XXX duplicates the above
+        Just (AtDefault _mc) | multiperiod -> ", valued at period ends"
+        Just (AtDefault _mc) -> ", current value"
+        Just (AtDate d _mc) -> ", valued at "++showDate d
+        Nothing             -> "")
 
 -- | Build a 'Table' from a multi-column balance report.
 balanceReportAsTable :: ReportOpts -> MultiBalanceReport -> Table String String MixedAmount
-balanceReportAsTable opts@ReportOpts{average_, row_total_} (MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
+balanceReportAsTable opts@ReportOpts{average_, row_total_, balancetype_} (MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
    maybetranspose $
-   addtotalrow $ 
+   addtotalrow $
    Table
      (T.Group NoLine $ map Header accts)
      (T.Group NoLine $ map Header colheadings)
      (map rowvals items)
   where
-    mkDate = case balancetype_ opts of
+    totalscolumn = row_total_ && not (balancetype_ `elem` [CumulativeChange, HistoricalBalance])
+    mkDate = case balancetype_ of
        PeriodChange -> showDateSpanMonthAbbrev
        _            -> maybe "" (showDate . prevday) . spanEnd
     colheadings = map mkDate colspans
-                  ++ ["  Total" | row_total_]
+                  ++ ["  Total" | totalscolumn]
                   ++ ["Average" | average_]
     accts = map renderacct items
     renderacct (a,a',i,_,_,_)
       | tree_ opts = replicate ((i-1)*2) ' ' ++ T.unpack a'
       | otherwise  = T.unpack $ maybeAccountNameDrop opts a
     rowvals (_,_,_,as,rowtot,rowavg) = as
-                             ++ [rowtot | row_total_]
+                             ++ [rowtot | totalscolumn]
                              ++ [rowavg | average_]
     addtotalrow | no_total_ opts = id
                 | otherwise      = (+----+ (row "" $
                                     coltotals
-                                    ++ [tot | row_total_ && not (null coltotals)]
+                                    ++ [tot | totalscolumn && not (null coltotals)]
                                     ++ [avg | average_   && not (null coltotals)]
                                     ))
     maybetranspose | transpose_ opts = \(Table rh ch vals) -> Table ch rh (transpose vals)
                    | otherwise       = id
-                   
+
 -- | Given a table representing a multi-column balance report (for example,
 -- made using 'balanceReportAsTable'), render it in a format suitable for
 -- console output.
@@ -638,15 +641,16 @@ tests_Balance = tests "Balance" [
 
    tests "balanceReportAsText" [
     test "unicode in balance layout" $ do
-      j <- io $ readJournal' "2009/01/01 * медвежья шкура\n  расходы:покупки  100\n  актив:наличные\n"
+      j <- readJournal' "2009/01/01 * медвежья шкура\n  расходы:покупки  100\n  актив:наличные\n"
       let opts = defreportopts
-      balanceReportAsText opts (balanceReport opts (queryFromOpts (parsedate "2008/11/26") opts) j) `is`
+      balanceReportAsText opts (balanceReport opts (queryFromOpts (parsedate "2008/11/26") opts) j)
+        @?=
         unlines
         ["                -100  актив:наличные"
         ,"                 100  расходы:покупки"
         ,"--------------------"
         ,"                   0"
         ]
-  ]
+    ]
 
- ]
+  ]
