@@ -4,7 +4,7 @@
 module Hledger.Cli.Commands.Import (
   importmode
  ,importcmd
-) 
+)
 where
 
 import Control.Monad
@@ -18,14 +18,19 @@ import Text.Printf
 
 importmode = hledgerCommandMode
   $(embedFileRelative "Hledger/Cli/Commands/Import.txt")
-  [flagNone ["dry-run"] (setboolopt "dry-run") "just show the transactions to be imported"] 
+  [flagNone ["catchup"] (setboolopt "catchup") "just mark all transactions as already imported"
+  ,flagNone ["dry-run"] (setboolopt "dry-run") "just show the transactions to be imported"
+  ]
   [generalflagsgroup1]
-  []
+  hiddenflags
   ([], Just $ argsFlag "FILE [...]")
 
 importcmd opts@CliOpts{rawopts_=rawopts,inputopts_=iopts} j = do
+  -- XXX could be helpful to show the last-seen date, and number of old transactions, too
   let
     inputfiles = listofstringopt "args" rawopts
+    inputstr = intercalate ", " $ map quoteIfNeeded inputfiles
+    catchup = boolopt "catchup" rawopts
     dryrun = boolopt "dry-run" rawopts
     iopts' = iopts{new_=True, new_save_=not dryrun}
   case inputfiles of
@@ -33,15 +38,21 @@ importcmd opts@CliOpts{rawopts_=rawopts,inputopts_=iopts} j = do
     fs -> do
       enewj <- readJournalFiles iopts' fs
       case enewj of
-        Left e     -> error' e 
+        Left e     -> error' e
         Right newj ->
           case sortOn tdate $ jtxns newj of
-            [] -> return ()
+            -- with --dry-run the output should be valid journal format, so messages have ; prepended
+            [] -> do
+              -- in this case, we vary the output depending on --dry-run, which is a bit awkward
+              let semicolon = if dryrun then "; " else "" :: String
+              printf "%sno new transactions found in %s\n\n" semicolon inputstr
             newts | dryrun -> do
-              printf "; would import %d new transactions:\n\n" (length newts)
+              printf "; would import %d new transactions from %s:\n\n" (length newts) inputstr
               -- TODO how to force output here ?
               -- length (jtxns newj) `seq` print' opts{rawopts_=("explicit",""):rawopts} newj
-              mapM_ (putStr . showTransactionUnelided) newts
+              mapM_ (putStr . showTransaction) newts
+            newts | catchup -> do
+              printf "marked %s as caught up, skipping %d unimported transactions\n\n" inputstr (length newts)
             newts -> do
               foldM_ (`journalAddTransaction` opts) j newts  -- gets forced somehow.. (how ?)
-              printf "imported %d new transactions\n" (length newts)
+              printf "imported %d new transactions from %s\n" (length newts) inputstr
