@@ -37,7 +37,7 @@ printmode = hledgerCommandMode
     "show only newer-dated transactions added in each file since last run"
   ] ++ outputflags)
   [generalflagsgroup1]
-  []
+  hiddenflags
   ([], Just $ argsFlag "[QUERY]")
 
 -- | Print journal transactions in standard format.
@@ -59,14 +59,14 @@ printEntries opts@CliOpts{reportopts_=ropts} j = do
   writeOutput opts $ render $ entriesReport ropts' q j
 
 entriesReportAsText :: CliOpts -> EntriesReport -> String
-entriesReportAsText opts = concatMap (showTransactionUnelided . gettxn) 
+entriesReportAsText opts = concatMap (showTransaction . gettxn)
   where
-    gettxn | useexplicittxn = id                   -- use fully inferred amounts & txn prices 
+    gettxn | useexplicittxn = id                   -- use fully inferred amounts & txn prices
            | otherwise      = originalTransaction  -- use original as-written amounts/txn prices
     -- Original vs inferred transactions/postings were causing problems here, disabling -B (#551).
     -- Use the explicit one if -B or -x are active.
-    -- This passes tests; does it also mean -B sometimes shows missing amounts unnecessarily ?  
-    useexplicittxn = boolopt "explicit" (rawopts_ opts) || cost_ (reportopts_ opts)
+    -- This passes tests; does it also mean -B sometimes shows missing amounts unnecessarily ?
+    useexplicittxn = boolopt "explicit" (rawopts_ opts) || (valuationTypeIsCost $ reportopts_ opts)
 
 -- Replace this transaction's postings with the original postings if any, but keep the
 -- current possibly rewritten account names.
@@ -162,7 +162,7 @@ printMatch CliOpts{reportopts_=ropts} j desc = do
   let q = queryFromOpts d ropts
   case similarTransaction' j q desc of
                 Nothing -> putStrLn "no matches found."
-                Just t  -> putStr $ showTransactionUnelided t
+                Just t  -> putStr $ showTransaction t
 
   where
     -- Identify the closest recent match for this description in past transactions.
