@@ -4,7 +4,9 @@ A ledger-compatible @register@ command.
 
 -}
 
-{-# LANGUAGE CPP, OverloadedStrings #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE TemplateHaskell #-}
 
 module Hledger.Cli.Commands.Register (
@@ -48,12 +50,12 @@ registermode = hledgerCommandMode
      )
   ] ++ outputflags)
   [generalflagsgroup1]
-  []
+  hiddenflags
   ([], Just $ argsFlag "[QUERY]")
 
 -- | Print a (posting) register report.
 register :: CliOpts -> Journal -> IO ()
-register opts@CliOpts{reportopts_=ropts} j = do
+register opts@CliOpts{reportopts_=ropts@ReportOpts{..}} j = do
   d <- getCurrentDay
   let fmt = outputFormatFromOpts opts
       render | fmt=="csv"  = const ((++"\n") . printCSV . postingsReportAsCsv)
@@ -173,8 +175,10 @@ postingsReportItemAsText opts preferredamtwidth preferredbalwidth (mdate, mendda
               BalancedVirtualPosting -> (\s -> "["++s++"]", acctwidth-2)
               VirtualPosting         -> (\s -> "("++s++")", acctwidth-2)
               _                      -> (id,acctwidth)
-      amt = showMixedAmountWithoutPrice $ pamount p
-      bal = showMixedAmountWithoutPrice b
+      showamt | color_ (reportopts_ opts) = cshowMixedAmountWithoutPrice
+              | otherwise                 = showMixedAmountWithoutPrice
+      amt = showamt $ pamount p
+      bal = showamt b
       -- alternate behaviour, show null amounts as 0 instead of blank
       -- amt = if null amt' then "0" else amt'
       -- bal = if null bal' then "0" else bal'
@@ -191,9 +195,11 @@ tests_Register = tests "Register" [
 
    tests "postingsReportAsText" [
     test "unicode in register layout" $ do
-      j <- io $ readJournal' "2009/01/01 * медвежья шкура\n  расходы:покупки  100\n  актив:наличные\n"
+      j <- readJournal' "2009/01/01 * медвежья шкура\n  расходы:покупки  100\n  актив:наличные\n"
       let opts = defreportopts
-      (postingsReportAsText defcliopts $ postingsReport opts (queryFromOpts (parsedate "2008/11/26") opts) j) `is` unlines
+      (postingsReportAsText defcliopts $ postingsReport opts (queryFromOpts (parsedate "2008/11/26") opts) j)
+        @?=
+        unlines
         ["2009/01/01 медвежья шкура       расходы:покупки                100           100"
         ,"                                актив:наличные                -100             0"]
    ]
