@@ -35,7 +35,7 @@ statsmode = hledgerCommandMode
   [flagReq  ["output-file","o"]   (\s opts -> Right $ setopt "output-file" s opts) "FILE" "write output to FILE."
   ]
   [generalflagsgroup1]
-  []
+  hiddenflags
   ([], Just $ argsFlag "[QUERY]")
 
 -- like Register.summarisePostings
@@ -70,6 +70,7 @@ showLedgerStats l today span =
         ,("Payees/descriptions", show $ size $ fromList $ map (tdescription) ts)
         ,("Accounts", printf "%d (depth %d)" acctnum acctdepth)
         ,("Commodities", printf "%s (%s)" (show $ length cs) (T.intercalate ", " cs))
+        ,("Market prices", printf "%s (%s)" (show $ length mktprices) (T.intercalate ", " mktpricecommodities))
       -- Transactions this month     : %(monthtxns)s (last month in the same period: %(lastmonthtxns)s)
       -- Unmarked transactions      : %(unmarked)s
       -- Days since reconciliation   : %(reconcileelapsed)s
@@ -80,7 +81,7 @@ showLedgerStats l today span =
              path = journalFilePath j
              ts = sortOn tdate $ filter (spanContainsDate span . tdate) $ jtxns j
              as = nub $ map paccount $ concatMap tpostings ts
-             cs = Map.keys $ commodityStylesFromAmounts $ concatMap (amounts . pamount) $ concatMap tpostings ts
+             cs = either error' Map.keys $ commodityStylesFromAmounts $ concatMap (amounts . pamount) $ concatMap tpostings ts
              lastdate | null ts = Nothing
                       | otherwise = Just $ tdate $ last ts
              lastelapsed = fmap (diffDays today) lastdate
@@ -106,4 +107,5 @@ showLedgerStats l today span =
              acctnum = length as
              acctdepth | null as = 0
                        | otherwise = maximum $ map accountNameLevel as
-
+             mktprices = jpricedirectives j
+             mktpricecommodities = nub $ sort $ map pdcommodity mktprices
