@@ -1,7 +1,7 @@
-{-# LANGUAGE OverloadedStrings, RecordWildCards #-}
+{-# LANGUAGE OverloadedStrings, RecordWildCards, LambdaCase #-}
 {-|
 
-Common helpers for making multi-section balance report commands 
+Common helpers for making multi-section balance report commands
 like balancesheet, cashflow, and incomestatement.
 
 -}
@@ -14,12 +14,14 @@ module Hledger.Cli.CompoundBalanceCommand (
 ) where
 
 import Data.List (foldl')
-import Data.Maybe (fromMaybe,catMaybes)
+import Data.Maybe
 import qualified Data.Text as TS
 import qualified Data.Text.Lazy as TL
+import Data.Time.Calendar
+import Data.Time.Format
 import System.Console.CmdArgs.Explicit as C
 import Hledger.Read.CsvReader (CSV, printCSV)
-import Lucid as L
+import Lucid as L hiding (value_)
 import Text.Tabular as T
 
 import Hledger
@@ -27,16 +29,16 @@ import Hledger.Cli.Commands.Balance
 import Hledger.Cli.CliOptions
 import Hledger.Cli.Utils (writeOutput)
 
--- | Description of a compound balance report command, 
+-- | Description of a compound balance report command,
 -- from which we generate the command's cmdargs mode and IO action.
--- A compound balance report command shows one or more sections/subreports, 
--- each with its own title and subtotals row, in a certain order, 
+-- A compound balance report command shows one or more sections/subreports,
+-- each with its own title and subtotals row, in a certain order,
 -- plus a grand totals row if there's more than one section.
 -- Examples are the balancesheet, cashflow and incomestatement commands.
 --
--- Compound balance reports do sign normalisation: they show all account balances 
+-- Compound balance reports do sign normalisation: they show all account balances
 -- as normally positive, unlike the ordinary BalanceReport and most hledger commands
--- which show income/liability/equity balances as normally negative.  
+-- which show income/liability/equity balances as normally negative.
 -- Each subreport specifies the normal sign of its amounts, and whether
 -- it should be added to or subtracted from the grand total.
 --
@@ -44,7 +46,7 @@ data CompoundBalanceCommandSpec = CompoundBalanceCommandSpec {
   cbcdoc      :: CommandDoc,          -- ^ the command's name(s) and documentation
   cbctitle    :: String,              -- ^ overall report title
   cbcqueries  :: [CBCSubreportSpec],  -- ^ subreport details
-  cbctype     :: BalanceType          -- ^ the "balance" type (change, cumulative, historical) 
+  cbctype     :: BalanceType          -- ^ the "balance" type (change, cumulative, historical)
                                       --   this report shows (overrides command line flags)
 }
 
@@ -62,15 +64,15 @@ data CBCSubreportSpec = CBCSubreportSpec {
 --
 -- * the period (date span) of each column
 --
--- * one or more named, normal-positive multi balance reports, 
+-- * one or more named, normal-positive multi balance reports,
 --   with columns corresponding to the above, and a flag indicating
 --   whether they increased or decreased the overall totals
 --
 -- * a list of overall totals for each column, and their grand total and average
 --
--- It is used in compound balance report commands like balancesheet, 
+-- It is used in compound balance report commands like balancesheet,
 -- cashflow and incomestatement.
-type CompoundBalanceReport = 
+type CompoundBalanceReport =
   ( String
   , [DateSpan]
   , [(String, MultiBalanceReport, Bool)]
@@ -78,7 +80,7 @@ type CompoundBalanceReport =
   )
 
 
--- | Generate a cmdargs option-parsing mode from a compound balance command 
+-- | Generate a cmdargs option-parsing mode from a compound balance command
 -- specification.
 compoundBalanceCommandMode :: CompoundBalanceCommandSpec -> Mode RawOpts
 compoundBalanceCommandMode CompoundBalanceCommandSpec{..} =
@@ -104,11 +106,12 @@ compoundBalanceCommandMode CompoundBalanceCommandSpec{..} =
     ,flagReq  ["format"] (\s opts -> Right $ setopt "format" s opts) "FORMATSTR" "use this custom line format (in simple reports)"
     ,flagNone ["pretty-tables"] (setboolopt "pretty-tables") "use unicode when displaying tables"
     ,flagNone ["sort-amount","S"] (setboolopt "sort-amount") "sort by amount instead of account code/name"
+    ,flagNone ["percent", "%"] (setboolopt "percent") "express values in percentage of each column's total"
     ,outputFormatFlag
     ,outputFileFlag
     ]
     [generalflagsgroup1]
-    []
+    hiddenflags
     ([], Just $ argsFlag "[QUERY]")
  where
    defType :: BalanceType -> String
@@ -117,79 +120,112 @@ compoundBalanceCommandMode CompoundBalanceCommandSpec{..} =
 
 -- | Generate a runnable command from a compound balance command specification.
 compoundBalanceCommand :: CompoundBalanceCommandSpec -> (CliOpts -> Journal -> IO ())
-compoundBalanceCommand CompoundBalanceCommandSpec{..} opts@CliOpts{reportopts_=ropts, rawopts_=rawopts} j = do
+compoundBalanceCommand CompoundBalanceCommandSpec{..} opts@CliOpts{reportopts_=ropts@ReportOpts{..}, rawopts_=rawopts} j = do
     d <- getCurrentDay
     let
-      -- use the default balance type for this report, unless the user overrides  
+      -- use the default balance type for this report, unless the user overrides
       mBalanceTypeOverride =
-        case reverse $ filter (`elem` ["change","cumulative","historical"]) $ map fst rawopts of
-          "historical":_ -> Just HistoricalBalance
-          "cumulative":_ -> Just CumulativeChange
-          "change":_     -> Just PeriodChange
-          _              -> Nothing
+        choiceopt parse rawopts where
+          parse = \case
+            "historical" -> Just HistoricalBalance
+            "cumulative" -> Just CumulativeChange
+            "change"     -> Just PeriodChange
+            _            -> Nothing
       balancetype = fromMaybe cbctype mBalanceTypeOverride
-      title = cbctitle ++ " " ++ showDateSpan requestedspan ++ maybe "" (' ':) mtitleclarification
-        where
-          requestedspan = queryDateSpan (date2_ ropts) userq `spanDefaultsFrom` journalDateSpan (date2_ ropts) j
-          -- when user overrides, add an indication to the report title
-          mtitleclarification = flip fmap mBalanceTypeOverride $ \t ->
-            case t of
-              PeriodChange      -> "(Balance Changes)"
-              CumulativeChange  -> "(Cumulative Ending Balances)"
-              HistoricalBalance -> "(Historical Ending Balances)"
       -- Set balance type in the report options.
-      -- Also, use tree mode (by default, at least?) if --cumulative/--historical 
-      -- are used in single column mode, since in that situation we will be using 
+      -- Also, use tree mode (by default, at least?) if --cumulative/--historical
+      -- are used in single column mode, since in that situation we will be using
       -- balanceReportFromMultiBalanceReport which does not support eliding boring parents,
-      -- and tree mode hides this.. or something.. XXX 
-      ropts'
-        | not (flat_ ropts) && 
-          interval_ ropts==NoInterval && 
-          balancetype `elem` [CumulativeChange, HistoricalBalance]
-            = ropts{balancetype_=balancetype, accountlistmode_=ALTree}
-        | otherwise
-            = ropts{balancetype_=balancetype}
+      -- and tree mode hides this.. or something.. XXX
+      ropts' = ropts{
+        balancetype_=balancetype,
+        accountlistmode_=if not (flat_ ropts) && interval_==NoInterval && balancetype `elem` [CumulativeChange, HistoricalBalance] then ALTree else accountlistmode_,
+        no_total_=if percent_ && length cbcqueries > 1 then True else no_total_
+      }
       userq = queryFromOpts d ropts'
       format = outputFormatFromOpts opts
 
-      -- make a CompoundBalanceReport
-      subreports = 
-        map (\CBCSubreportSpec{..} -> 
+      -- make a CompoundBalanceReport.
+      -- For efficiency, generate a price oracle here and reuse it with each subreport.
+      priceoracle = journalPriceOracle j
+      subreports =
+        map (\CBCSubreportSpec{..} ->
                 (cbcsubreporttitle
                 ,mbrNormaliseSign cbcsubreportnormalsign $ -- <- convert normal-negative to normal-positive
-                  compoundBalanceSubreport ropts' userq j cbcsubreportquery cbcsubreportnormalsign
+                  compoundBalanceSubreport ropts' userq j priceoracle cbcsubreportquery cbcsubreportnormalsign
                 ,cbcsubreportincreasestotal
                 ))
             cbcqueries
-      subtotalrows = 
-        [(coltotals, increasesoveralltotal) 
+
+      subtotalrows =
+        [(coltotals, increasesoveralltotal)
         | (_, MultiBalanceReport (_,_,(coltotals,_,_)), increasesoveralltotal) <- subreports
         ]
+
       -- Sum the subreport totals by column. Handle these cases:
       -- - no subreports
       -- - empty subreports, having no subtotals (#588)
-      -- - subreports with a shorter subtotals row than the others  
+      -- - subreports with a shorter subtotals row than the others
       overalltotals = case subtotalrows of
         [] -> ([], nullmixedamt, nullmixedamt)
         rs ->
           let
             numcols = maximum $ map (length.fst) rs  -- partial maximum is ok, rs is non-null
-            paddedsignedsubtotalrows = 
+            paddedsignedsubtotalrows =
               [map (if increasesoveralltotal then id else negate) $  -- maybe flip the signs
-               take numcols $ as ++ repeat nullmixedamt              -- pad short rows with zeros 
+               take numcols $ as ++ repeat nullmixedamt              -- pad short rows with zeros
               | (as,increasesoveralltotal) <- rs
               ]
             coltotals = foldl' (zipWith (+)) zeros paddedsignedsubtotalrows  -- sum the columns
               where zeros = replicate numcols nullmixedamt
             grandtotal = sum coltotals
             grandavg | null coltotals = nullmixedamt
-                     | otherwise      = fromIntegral (length coltotals) `divideMixedAmount` grandtotal 
-          in 
+                     | otherwise      = fromIntegral (length coltotals) `divideMixedAmount` grandtotal
+          in
             (coltotals, grandtotal, grandavg)
+
       colspans =
         case subreports of
           (_, MultiBalanceReport (ds,_,_), _):_ -> ds
           [] -> []
+
+      title =
+        cbctitle
+        ++ " "
+        ++ titledatestr
+        ++ maybe "" (' ':) mtitleclarification
+        ++ valuationdesc
+        where
+
+          -- XXX #1078 the title of ending balance reports
+          -- (HistoricalBalance) should mention the end date(s) shown as
+          -- column heading(s) (not the date span of the transactions).
+          -- Also the dates should not be simplified (it should show
+          -- "2008/01/01-2008/12/31", not "2008").
+          titledatestr
+            | balancetype == HistoricalBalance = showEndDates enddates
+            | otherwise                        = showDateSpan requestedspan 
+            where
+              enddates = map (addDays (-1)) $ catMaybes $ map spanEnd colspans  -- these spans will always have a definite end date
+              requestedspan = queryDateSpan date2_ userq `spanDefaultsFrom` journalDateSpan date2_ j
+
+          -- when user overrides, add an indication to the report title
+          mtitleclarification = flip fmap mBalanceTypeOverride $ \t ->
+            case t of
+              PeriodChange      -> "(Balance Changes)"
+              CumulativeChange  -> "(Cumulative Ending Balances)"
+              HistoricalBalance -> "(Historical Ending Balances)"
+
+          valuationdesc = case value_ of
+            Just (AtCost _mc)   -> ", valued at cost"
+            Just (AtEnd _mc)    -> ", valued at period ends"
+            Just (AtNow _mc)    -> ", current value"
+            Just (AtDefault _mc) | multiperiod   -> ", valued at period ends"
+            Just (AtDefault _mc)    -> ", current value"
+            Just (AtDate d _mc) -> ", valued at "++showDate d
+            Nothing             -> ""
+            where
+              multiperiod = interval_ /= NoInterval
       cbr =
         (title
         ,colspans
@@ -204,25 +240,37 @@ compoundBalanceCommand CompoundBalanceCommandSpec{..} opts@CliOpts{reportopts_=r
         "html" -> (++ "\n") $ TL.unpack $ L.renderText $ compoundBalanceReportAsHtml ropts cbr
         _      -> compoundBalanceReportAsText ropts' cbr
 
+-- | Summarise one or more (inclusive) end dates, in a way that's
+-- visually different from showDateSpan, suggesting discrete end dates
+-- rather than a continuous span.
+showEndDates :: [Day] -> String
+showEndDates es = case es of
+  -- cf showPeriod
+  (e:_:_) -> showdate e ++ ",," ++ showdate (last es)
+  [e]     -> showdate e
+  []      -> ""
+  where
+    showdate = formatTime defaultTimeLocale "%0C%y/%m/%d" 
+
 -- | Run one subreport for a compound balance command in multi-column mode.
 -- This returns a MultiBalanceReport.
-compoundBalanceSubreport :: ReportOpts -> Query -> Journal -> (Journal -> Query) -> NormalSign -> MultiBalanceReport
-compoundBalanceSubreport ropts userq j subreportqfn subreportnormalsign = r'
+compoundBalanceSubreport :: ReportOpts -> Query -> Journal -> PriceOracle -> (Journal -> Query) -> NormalSign -> MultiBalanceReport
+compoundBalanceSubreport ropts@ReportOpts{..} userq j priceoracle subreportqfn subreportnormalsign = r'
   where
     -- force --empty to ensure same columns in all sections
     ropts' = ropts { empty_=True, normalbalance_=Just subreportnormalsign }
     -- run the report
     q = And [subreportqfn j, userq]
-    r@(MultiBalanceReport (dates, rows, totals)) = multiBalanceReport ropts' q j
+    r@(MultiBalanceReport (dates, rows, totals)) = multiBalanceReportWith ropts' q j priceoracle
     -- if user didn't specify --empty, now remove the all-zero rows, unless they have non-zero subaccounts
     -- in this report
-    r' | empty_ ropts = r
-       | otherwise    = MultiBalanceReport (dates, rows', totals) 
+    r' | empty_    = r
+       | otherwise = MultiBalanceReport (dates, rows', totals)
           where
             nonzeroaccounts =
               dbg1 "nonzeroaccounts" $
               catMaybes $ map (\(act,_,_,amts,_,_) ->
-                            if not (all isZeroMixedAmount amts) then Just act else Nothing) rows 
+                            if not (all isZeroMixedAmount amts) then Just act else Nothing) rows
             rows' = filter (not . emptyRow) rows
               where
                 emptyRow (act,_,_,amts,_,_) =
@@ -232,34 +280,34 @@ compoundBalanceSubreport ropts userq j subreportqfn subreportnormalsign = r'
 {- Eg:
 Balance Sheet
 
-             ||  2017/12/31    Total  Average 
+             ||  2017/12/31    Total  Average
 =============++===============================
- Assets      ||                               
+ Assets      ||
 -------------++-------------------------------
- assets:b    ||           1        1        1 
+ assets:b    ||           1        1        1
 -------------++-------------------------------
-             ||           1        1        1 
+             ||           1        1        1
 =============++===============================
- Liabilities ||                               
+ Liabilities ||
 -------------++-------------------------------
 -------------++-------------------------------
-             ||                               
+             ||
 =============++===============================
- Total       ||           1        1        1 
+ Total       ||           1        1        1
 
 -}
 compoundBalanceReportAsText :: ReportOpts -> CompoundBalanceReport -> String
 compoundBalanceReportAsText ropts (title, _colspans, subreports, (coltotals, grandtotal, grandavg)) =
-  title ++ "\n\n" ++ 
+  title ++ "\n\n" ++
   balanceReportTableAsText ropts bigtable'
   where
     singlesubreport = length subreports == 1
-    bigtable = 
+    bigtable =
       case map (subreportAsTable ropts singlesubreport) subreports of
         []   -> T.empty
         r:rs -> foldl' concatTables r rs
     bigtable'
-      | no_total_ ropts || singlesubreport = 
+      | no_total_ ropts || singlesubreport =
           bigtable
       | otherwise =
           bigtable
@@ -319,11 +367,11 @@ compoundBalanceReportAsCsv ropts (title, colspans, subreports, (coltotals, grand
             (if row_total_ ropts then (1+) else id) $
             (if average_ ropts then (1+) else id) $
             maximum $ -- depends on non-null subreports
-            map (\(MultiBalanceReport (amtcolheadings, _, _)) -> length amtcolheadings) $ 
+            map (\(MultiBalanceReport (amtcolheadings, _, _)) -> length amtcolheadings) $
             map second3 subreports
     addtotals
       | no_total_ ropts || length subreports == 1 = id
-      | otherwise = (++ 
+      | otherwise = (++
           ["Net:" :
            map showMixedAmountOneLineWithoutPrice (
              coltotals
@@ -337,7 +385,7 @@ compoundBalanceReportAsHtml :: ReportOpts -> CompoundBalanceReport -> Html ()
 compoundBalanceReportAsHtml ropts cbr =
   let
     (title, colspans, subreports, (coltotals, grandtotal, grandavg)) = cbr
-    colspanattr = colspan_ $ TS.pack $ show $ 
+    colspanattr = colspan_ $ TS.pack $ show $
       1 + length colspans + (if row_total_ ropts then 1 else 0) + (if average_ ropts then 1 else 0)
     leftattr = style_ "text-align:left"
     blankrow = tr_ $ td_ [colspanattr] $ toHtmlRaw ("&nbsp;"::String)
@@ -353,7 +401,7 @@ compoundBalanceReportAsHtml ropts cbr =
 
     thRow :: [String] -> Html ()
     thRow = tr_ . mconcat . map (th_ . toHtml)
-    
+
     -- Make rows for a subreport: its title row, not the headings row,
     -- the data rows, any totals row, and a blank row for whitespace.
     subreportrows :: (String, MultiBalanceReport, Bool) -> [Html ()]
