@@ -43,7 +43,7 @@ module Hledger.Cli.Main where
 import Data.Char (isDigit)
 import Data.List
 import Safe
-import System.Console.CmdArgs.Explicit as C
+import qualified System.Console.CmdArgs.Explicit as C
 import System.Environment
 import System.Exit
 import System.FilePath
@@ -156,6 +156,7 @@ main = do
   dbgIO "interval from opts" (interval_ $ reportopts_ opts)
   dbgIO "query from opts & args" (queryFromOpts d $ reportopts_ opts)
   let
+    journallesserror = error "journal-less command tried to use the journal"
     runHledgerCommand
       -- high priority flags and situations. -h, then --help, then --info are highest priority.
       | hasHelpFlag argsbeforecmd = dbgIO "" "-h before command, showing general usage" >> printUsage
@@ -170,14 +171,15 @@ main = do
 
       -- builtin commands
       | Just (cmdmode, cmdaction) <- findCommand cmd =
-        (case cmd of
-          "test" -> -- should not read the journal
-            cmdaction opts (error "journal-less command tried to use the journal")
-          "add" ->  -- should create the journal if missing
-            (ensureJournalFileExists =<< (head <$> journalFilePathFromOpts opts)) >>
-            withJournalDo opts (cmdaction opts)
-          _ ->      -- all other commands: read the journal or fail if missing
+        (case True of
+           -- these commands should not require or read the journal
+          _ | cmd `elem` ["test","help"] -> cmdaction opts journallesserror
+          -- these commands should create the journal if missing
+          _ | cmd `elem` ["add","import"] -> do
+            (ensureJournalFileExists =<< (head <$> journalFilePathFromOpts opts))
             withJournalDo opts (cmdaction opts)
+          -- other commands read the journal and should fail if it's missing
+          _ -> withJournalDo opts (cmdaction opts)
         )
         `orShowHelp` cmdmode
 
@@ -204,9 +206,8 @@ argsToCliOpts :: [String] -> [String] -> IO CliOpts
 argsToCliOpts args addons = do
   let
     args'        = moveFlagsAfterCommand $ replaceNumericFlags args
-    cmdargsopts  = either usageError id $ process (mainmode addons) args'
-    cmdargsopts' = decodeRawOpts cmdargsopts
-  rawOptsToCliOpts cmdargsopts'
+    cmdargsopts  = either usageError id $ C.process (mainmode addons) args'
+  rawOptsToCliOpts cmdargsopts
 
 -- | A hacky workaround for cmdargs not accepting flags before the
 -- subcommand name: try to detect and move such flags after the
