@@ -1,5 +1,6 @@
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE NoOverloadedStrings #-} -- prevent trouble if turned on in ghci
 {-|
 
 Utilities for top-level modules and ghci. See also Hledger.Read and
@@ -12,7 +13,6 @@ module Hledger.Cli.Utils
      withJournalDo,
      writeOutput,
      journalTransform,
-     journalApplyValue,
      journalAddForecast,
      journalReload,
      journalReloadIfChanged,
@@ -25,19 +25,17 @@ module Hledger.Cli.Utils
      readFileStrictly,
      pivotByOpts,
      anonymiseByOpts,
+     tests_Cli_Utils,
     )
 where
 import Control.Exception as C
 import Control.Monad
 
-import Data.Hashable (hash)
 import Data.List
 import Data.Maybe
 import qualified Data.Text as T
 import qualified Data.Text.IO as T
 import Data.Time (Day, addDays)
-import Data.Word
-import Numeric
 import Safe (readMay)
 import System.Console.CmdArgs
 import System.Directory (getModificationTime, getDirectoryContents, copyFile)
@@ -53,13 +51,14 @@ import System.Time (ClockTime(TOD))
 import Data.Time.Clock.POSIX (utcTimeToPOSIXSeconds)
 
 import Hledger.Cli.CliOptions
+import Hledger.Cli.Anon
 import Hledger.Data
 import Hledger.Read
 import Hledger.Reports
 import Hledger.Utils
 
 -- | Parse the user's specified journal file(s) as a Journal, maybe apply some
--- transformations according to options, and run a hledger command with it. 
+-- transformations according to options, and run a hledger command with it.
 -- Or, throw an error.
 withJournalDo :: CliOpts -> (Journal -> IO a) -> IO a
 withJournalDo opts cmd = do
@@ -71,17 +70,19 @@ withJournalDo opts cmd = do
   >>= mapM (journalTransform opts)
   >>= either error' cmd
 
--- | Apply some transformations to the journal if specified by options.
--- These include:
+-- | Apply some extra post-parse transformations to the journal, if
+-- specified by options. These happen after journal validation, but
+-- before report calculation. They include:
 --
 -- - adding forecast transactions (--forecast)
--- - converting amounts to market value (--value)
 -- - pivoting account names (--pivot)
 -- - anonymising (--anonymise).
+--
 journalTransform :: CliOpts -> Journal -> IO Journal
-journalTransform opts@CliOpts{reportopts_=ropts} =
+journalTransform opts@CliOpts{reportopts_=_ropts} =
       journalAddForecast opts
-  >=> journalApplyValue ropts
+-- - converting amounts to market value (--value)
+  -- >=> journalApplyValue ropts
   >=> return . pivotByOpts opts
   >=> return . anonymiseByOpts opts
 
@@ -95,57 +96,25 @@ pivotByOpts opts =
 -- | Apply the anonymisation transformation on a journal, if option is present
 anonymiseByOpts :: CliOpts -> Journal -> Journal
 anonymiseByOpts opts =
-  case maybestringopt "anon" . rawopts_ $ opts of
-    Just _  -> anonymise
-    Nothing -> id
-
--- | Apply the anonymisation transformation on a journal
-anonymise :: Journal -> Journal
-anonymise j
-  = let
-      pAnons p = p { paccount = T.intercalate (T.pack ":") . map anon . T.splitOn (T.pack ":") . paccount $ p
-                   , pcomment = T.empty
-                   , ptransaction = fmap tAnons . ptransaction $ p
-                   , poriginal = pAnons <$> poriginal p
-                   }
-      tAnons txn = txn { tpostings = map pAnons . tpostings $ txn
-                       , tdescription = anon . tdescription $ txn
-                       , tcomment = T.empty
-                       }
-    in
-      j { jtxns = map tAnons . jtxns $ j }
-  where
-    anon = T.pack . flip showHex "" . (fromIntegral :: Int -> Word32) . hash
-
--- TODO move journalApplyValue and friends to Hledger.Data.Journal ? They are here because they use ReportOpts
-
--- | If -V/--value was requested, convert all journal amounts to their market value
--- as of the report end date. Cf http://hledger.org/manual.html#market-value
--- Since 2017/4 we do this early, before commands run, which affects all commands
--- and seems to have the same effect as doing it last on the reported values.
-journalApplyValue :: ReportOpts -> Journal -> IO Journal
-journalApplyValue ropts j = do
-    today <- getCurrentDay
-    mspecifiedenddate <- specifiedEndDate ropts
-    let d = fromMaybe today mspecifiedenddate
-        convert | value_ ropts = overJournalAmounts (amountValue j d)
-                | otherwise    = id
-    return $ convert j
+  if anon_ . inputopts_ $ opts
+      then anon
+      else id
 
 -- | Generate periodic transactions from all periodic transaction rules in the journal.
 -- These transactions are added to the in-memory Journal (but not the on-disk file).
 --
--- They start on or after the day following the latest normal transaction in the journal,
--- or today if there are none.
--- They end on or before the specified report end date, or 180 days from today if unspecified.
+-- They can start no earlier than: the day following the latest normal
+-- transaction in the journal (or today if there are none).
+-- They end on or before the specified report end date, or 180 days
+-- from today if unspecified.
 --
 journalAddForecast :: CliOpts -> Journal -> IO Journal
 journalAddForecast opts@CliOpts{inputopts_=iopts, reportopts_=ropts} j = do
   today <- getCurrentDay
 
-  -- "They start on or after the day following the latest normal transaction in the journal, or today if there are none."
-  let DateSpan _ mjournalend = dbg2 "journalspan"   $ journalDateSpan False j  -- ignore secondary dates
-      forecaststart          = dbg2 "forecaststart" $ fromMaybe today mjournalend
+  -- "They can start no earlier than: the day following the latest normal transaction in the journal (or today if there are none)."
+  let mjournalend   = dbg2 "journalEndDate" $ journalEndDate False j  -- ignore secondary dates
+      forecaststart = dbg2 "forecaststart" $ fromMaybe today mjournalend
 
   -- "They end on or before the specified report end date, or 180 days from today if unspecified."
   mspecifiedend <-  snd . dbg2 "specifieddates" <$> specifiedStartEndDates ropts
@@ -161,10 +130,10 @@ journalAddForecast opts@CliOpts{inputopts_=iopts, reportopts_=ropts} j = do
       forecasttxns' = (if auto_ iopts then modifyTransactions (jtxnmodifiers j) else id) forecasttxns
 
   return $
-    if forecast_ ropts 
+    if forecast_ ropts
       then journalBalanceTransactions' opts j{ jtxns = concat [jtxns j, forecasttxns'] }
       else j
-  where      
+  where
     journalBalanceTransactions' opts j =
       let assrt = not . ignore_assertions_ $ inputopts_ opts
       in
@@ -176,7 +145,7 @@ writeOutput :: CliOpts -> String -> IO ()
 writeOutput opts s = do
   f <- outputFileFromOpts opts
   (if f == "-" then putStr else writeFile f) s
-  
+
 -- -- | Get a journal from the given string and options, or throw an error.
 -- readJournal :: CliOpts -> String -> IO Journal
 -- readJournal opts s = readJournal def Nothing s >>= either error' return
@@ -231,7 +200,7 @@ fileModificationTime f
     | otherwise = (do
         utc <- getModificationTime f
         let nom = utcTimeToPOSIXSeconds utc
-        let clo = TOD (read $ takeWhile (`elem` "0123456789") $ show nom) 0 -- XXX read
+        let clo = TOD (read $ takeWhile (`elem` ("0123456789"::String)) $ show nom) 0 -- XXX read
         return clo
         )
         `C.catch` \(_::C.IOException) -> getClockTime
@@ -295,3 +264,30 @@ backupNumber :: FilePath -> FilePath -> Maybe Int
 backupNumber f g = case g =~ ("^" ++ f ++ "\\.([0-9]+)$") of
                         (_::FilePath, _::FilePath, _::FilePath, [ext::FilePath]) -> readMay ext
                         _ -> Nothing
+
+tests_Cli_Utils = tests "Utils" [
+
+  --  tests "journalApplyValue" [
+  --    -- Print the time required to convert one of the sample journals' amounts to value.
+  --    -- Pretty clunky, but working.
+  --    -- XXX sample.journal has no price records, but is always present.
+  --    -- Change to eg examples/5000x1000x10.journal to make this useful.
+  --    test "time" $ do
+  --      ej <- io $ readJournalFile definputopts "examples/3000x1000x10.journal"
+  --      case ej of
+  --        Left e  -> crash $ T.pack e
+  --        Right j -> do
+  --          (t,_) <- io $ timeItT $ do
+  --            -- Enable -V, and ensure the valuation date is later than
+  --            -- all prices for consistent timing.
+  --            let ropts = defreportopts{
+  --              value_=True,
+  --              period_=PeriodTo $ parsedate "3000-01-01"
+  --              }
+  --            j' <- journalApplyValue ropts j
+  --            sum (journalAmounts j') `seq` return ()
+  --          io $ printf "[%.3fs] " t
+  --          ok
+  -- ]
+
+ ]
